package config

type Config struct {
	Log   Log   `mapstructure:"log" json:"log" yaml:"log"`
	Serve Serve `mapstructure:"serve" json:"serve" yaml:"serve"`
	Redis Redis `mapstructure:"redis" json:"redis" yaml:"redis"`
}

// Serve gin服务配置
type Serve struct {
	Port               int      `mapstructure:"port" json:"port" yaml:"port" description:"gin开启监听http服务的端口"`
	RunMode            string   `mapstructure:"run-mode" json:"run-mode" yaml:"run-mode" description:"gin运行模式的默认，枚举值：debug,release"`
	UseRedis           bool     `mapstructure:"use-redis" json:"use-redis" yaml:"use-redis" description:"是否使用redis"`
	AppName            string   `mapstructure:"app-name" json:"app-name" yaml:"app-name" description:"项目名称"`
	StaticDir          string   `mapstructure:"static-dir" json:"static-dir" yaml:"static-dir" description:"上传的文件存储目录地址"`
	IndexSaveDir       string   `mapstructure:"index-save-dir" json:"index-save-dir" yaml:"index-save-dir" description:"监听文件夹的路径"`
	IndexMergeSaveDir  string   `mapstructure:"index-merge-save-dir" json:"index-merge-save-dir" yaml:"index-merge-save-dir" description:"监听合并文件夹的路径"`
	Frequency          string   `mapstructure:"frequency" json:"frequency" yaml:"frequency" description:"频度"`
	EdbLibUrl          string   `mapstructure:"edb-lib-url" json:"edb-lib-url" yaml:"edb-lib-url" description:"公共指标库的地址"`
	AppEdbLibNameEn    string   `mapstructure:"app_edb_lib_name_en" json:"app_edb_lib_name_en" yaml:"app_edb_lib_name_en" description:"指标库的英文名称"`
	EdbLibMd5Key       string   `mapstructure:"edb_lib_md5_key" json:"edb_lib_md5_key" yaml:"edb_lib_md5_key" description:"指标库服务秘钥"`
	TerminalCode       string   `mapstructure:"terminal_code" json:"terminal_code" yaml:"terminal_code" description:"终端编码"`
	IsCheckIndexUpdate bool     `mapstructure:"is_check_index_update" json:"is_check_index_update" yaml:"is_check_index_update" description:"是否检测指标更新状态"`
	RefreshTimeList    []string `mapstructure:"refresh_time_list" json:"refresh_time_list" yaml:"refresh_time_list" description:"刷新服务任务更新时间列表"`
}

// Log 日志配置
type Log struct {
	Prefix         string `mapstructure:"prefix" json:"prefix" yaml:"prefix" description:"日志输出前缀"`
	LogFile        bool   `mapstructure:"log-file" json:"logFile" yaml:"log-file" description:""`
	Stdout         string `mapstructure:"stdout" json:"stdout" yaml:"stdout" description:""`
	FileStdout     string `mapstructure:"file-stdout" json:"file-stdout" yaml:"file-stdout" description:""`
	SaveMaxDay     int    `mapstructure:"save-max-day" json:"save-max-day" yaml:"save-max-day" description:"最多保留多少天的日志"`
	CuttingDay     int    `mapstructure:"cutting-day" json:"cutting-day" yaml:"cutting-day" description:"相隔几天切割文件"`
	LogDirPath     string `mapstructure:"log-dir-path" json:"log-dir-path" yaml:"log-dir-path" description:"日志目录"`
	LogSoftLink    string `mapstructure:"log-soft-link" json:"log-soft-link" yaml:"log-soft-link" description:"日志软链接"`
	BinlogDirPath  string `mapstructure:"binlog-dir-path" json:"binlog-dir-path" yaml:"binlog-dir-path" description:"binlog日志目录"`
	BinlogSoftLink string `mapstructure:"binlog-soft-link" json:"binlog-soft-link" yaml:"binlog-soft-link" description:"binlog日志软链接"`
}

// Redis redis配置
type Redis struct {
	Address  string `mapstructure:"address" json:"address" yaml:"address" description:"redis服务链接地址"`
	Password string `mapstructure:"password" json:"password" yaml:"password" description:"redis服务密码"`
	Db       int    `mapstructure:"db" json:"db" yaml:"db" description:"默认使用的redis库"`
}
